# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

CONST_ACSTOR_ALL = "all"
CONST_ACSTOR_IO_ENGINE_LABEL_KEY = "acstor.azure.com/io-engine"
CONST_ACSTOR_IO_ENGINE_LABEL_VAL = "acstor"
CONST_ACSTOR_K8S_EXTENSION_NAME = "microsoft.azurecontainerstorage"
CONST_EXT_INSTALLATION_NAME = "azurecontainerstorage"
CONST_K8S_EXTENSION_CLIENT_FACTORY_MOD_NAME = "azext_k8s_extension._client_factory"
CONST_K8S_EXTENSION_CUSTOM_MOD_NAME = "azext_k8s_extension.custom"
CONST_K8S_EXTENSION_NAME = "k8s-extension"
CONST_STORAGE_POOL_DEFAULT_SIZE_ESAN = "1Ti"
CONST_STORAGE_POOL_DEFAULT_SIZE = "512Gi"
CONST_STORAGE_POOL_NAME_PREFIX = "storagepool-"
CONST_STORAGE_POOL_OPTION_NVME = "NVMe"
CONST_STORAGE_POOL_OPTION_SSD = "Temp"
CONST_STORAGE_POOL_SKU_PREMIUM_LRS = "Premium_LRS"
CONST_STORAGE_POOL_SKU_PREMIUM_ZRS = "Premium_ZRS"
CONST_STORAGE_POOL_SKU_PREMIUMV2_LRS = "PremiumV2_LRS"
CONST_STORAGE_POOL_SKU_STANDARD_LRS = "Standard_LRS"
CONST_STORAGE_POOL_SKU_STANDARDSSD_LRS = "StandardSSD_LRS"
CONST_STORAGE_POOL_SKU_STANDARDSSD_ZRS = "StandardSSD_ZRS"
CONST_STORAGE_POOL_SKU_ULTRASSD_LRS = "UltraSSD_LRS"
CONST_STORAGE_POOL_TYPE_AZURE_DISK = "azureDisk"
CONST_STORAGE_POOL_TYPE_ELASTIC_SAN = "elasticSan"
CONST_STORAGE_POOL_TYPE_EPHEMERAL_DISK = "ephemeralDisk"
